unit Demomain;

interface

{Demonstration program using TSQLBuilder component}
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, StdCtrls, DB, DBTables, SQLBuild, Grids, DBGrids,
  ExtCtrls, DBCtrls, Mask, Tabnotbk;

Const
  MaxDatabases = 4; {Set to whatever you like}
type
  TForm1 = class(TForm)
    Query1: TQuery;
    DataSource1: TDataSource;
    SQLBuilder1: TSQLBuilder;
    Panel2: TPanel;
    Label2: TLabel;
    Label3: TLabel;
    Memo1: TMemo;
    DBGrid1: TDBGrid;
    Panel1: TPanel;
    Bevel2: TBevel;
    Label4: TLabel;
    sbBuildSQL: TSpeedButton;
    sbRunQuery: TSpeedButton;
    SpeedButton1: TSpeedButton;
    Bevel1: TBevel;
    Label5: TLabel;
    rbQueryStartup: TRadioGroup;
    cboxChangeAlias: TCheckBox;
    cboxUpdateQueries: TCheckBox;
    cboxDeleteQueries: TCheckBox;
    cboxCanModify: TCheckBox;
    cboxAutoGroupby: TCheckBox;
    cboxShowSysTables: TCheckBox;
    cbSQLStyle: TComboBox;
    sbBlobDemo: TSpeedButton;
    procedure sbBuildSQLClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SpeedButton1Click(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure sbRunQueryClick(Sender: TObject);
    procedure sbBlobDemoClick(Sender: TObject);
  private
    { Private declarations }
    DBArray : Array[1..(MaxDatabases-1)] of TDatabase;
    StandardDB : TDatabase;
    Procedure SetBuilderOptions;
    Procedure PrepareAndRunQuery;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
Uses
  SqlBld08, blobmain;
{$R *.DFM}

function Long2Str(L : LongInt) : string;
  {-Convert a long/word/integer/byte/shortint to a string}
var
  S : string;
begin
  Str(L, S);
  Long2Str := S;
end;

Procedure Tform1.SetBuilderOptions;
Var
  TheOptions:TSQLBuilderOptions;
begin
  {Normally you would set all this in the component at design time}
  TheOptions := [sbEnablePrinting,sbEnableSaving];
  If cboxChangeAlias.Checked then
    Include(TheOptions, sbAllowChangeAlias);
  If cboxUpdateQueries.Checked then
    Include(TheOptions, sbAllowUpdates);
  If cboxDeleteQueries.Checked then
    Include(TheOptions, sbAllowDeletes);
  If cboxShowSysTables.Checked then
    Include(TheOptions, sbShowSystemTables);
  If cboxCanModify.Checked then
    Include(TheOptions, sbEnableEditor);
  SQLBuilder1.Options := TheOptions;
  SQLBuilder1.AutoGroup := cboxAutoGroupby.Checked;
  SQLBuilder1.AutoloadLastQuery := (rbQueryStartup.ItemIndex = 2);
  SQLBuilder1.UseWizardOnShow := (rbQueryStartup.ItemIndex = 1);
  SQLBuilder1.SQLType := TSQLType(cbSQLStyle.ItemIndex);
end;

procedure Tform1.PrepareAndRunQuery;
{Opens any necessary databases, sets up the TQuery and runs it}
Var
  ACount     :Integer;
begin
  If SQLBuilder1.QueryType <> SelectQuery then
    MessageDlg('Sorry, for safety reasons the demonstration program will only run "Select" queries.', mtInformation, [mbOk], 0)
  else If SQLBuilder1.UsedAliasList.Count > MaxDatabases then
    MessageDlg('Unable to run this query. The demo progam is limited to '
               +'a maximum of '+Long2Str(MaxDatabases)+' separate databases', mtInformation, [mbOk], 0)
  else
  Try
    screen.Cursor := crHourglass;

    {Place first alias in TQuery.Databasename}
    If SQLBuilder1.IsHeterogeneous then
      {use dummy program STANDARD alias as Databasename}
      Query1.DatabaseName := StandardDB.Databasename
    else
    If SQLBuilder1.UsedAliasList.Count > 0 then
      Query1.DatabaseName := SQLBuilder1.UsedAliasList.Strings[0];

    {If there is more than one alias involved then open databases for them}
    If SQLBuilder1.IsMultiAlias then
    For aCount := 1 to SQLBuilder1.UsedAliasList.Count-1 do
    Try
      DBArray[aCount].Free;
      DBArray[aCount] := TDatabase.Create(Self);
      DBArray[aCount].Aliasname := SQLBuilder1.UsedAliasList.Strings[aCount];
      DBArray[aCount].Databasename := SQLBuilder1.UsedAliasList.Strings[aCount];
      DBArray[aCount].Name := 'DB'+Long2Str(aCount+1);
      DBArray[aCount].Open;
    except
      On E: EDatabaseError do
      begin
        MessageDlg('Unable to open the database: '+DBArray[aCount].Aliasname
                   +#13#10+'Error: '+E.Message, mtError, [mbOk], 0);
        abort;
      end;
    end;

    {If we get this far then attempt to run the query}
    Query1.SQL.Assign(Memo1.Lines);
    If Query1.ParamCount > 0 then
      MessageDlg('Using replaceable parameters in queries is only supported within TSQLBuilder', mtInformation, [mbOk], 0)
    else
    Try
      Query1.Requestlive := True;
      Query1.Active := True;
    except
      Query1.Requestlive := False;
      Try
        Query1.Active := True;
      except
        On E: EDatabaseError do
          MessageDlg('Unable to run this query'+#13#10+'Error: '+E.Message, mtError, [mbOk], 0);
      end;
    end;
  finally
    screen.Cursor := crDefault;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  dCount:Integer;
begin
  {Initialize the Database array}
  For dCount := 1 to (MaxDatabases-1) do
    DBArray[dCount] := Nil;
  {Create a STANDARD type database alias to use with heterogeneous queries}
  StandardDB := TDatabase.Create(Self);
  StandardDB.Databasename := 'SQLBldDemo';
  StandardDB.Aliasname := Query1.Databasename;
  StandardDB.DriverName:= 'STANDARD';
  StandardDB.Params.Clear;
  StandardDB.Params.Add('PATH='+SQLBuilder1.PrivateDir);
  StandardDB.Open;
  cbSQLStyle.ItemIndex := 3;
end;

procedure TForm1.FormDestroy(Sender: TObject);
Var
  dCount:Integer;
begin
  StandardDB.Free;
  For dCount := 1 to (MaxDatabases-1) do
    DBArray[dCount].Free;
end;

procedure TForm1.sbBuildSQLClick(Sender: TObject);
begin
  Query1.Active := False;

  {Set SQLBuilder options to match options selected in checkboxes}
  SetBuilderOptions;

  {Build the query }
  If SQLBuilder1.Execute then
  begin
    If SQLBuilder1.SQLModified then
      memo1.lines.Assign(SQLBuilder1.UserSQL)
    else
      memo1.lines.Assign(SQLBuilder1.SQL);
    Application.Processmessages;
    PrepareAndRunQuery;
    rbQueryStartUp.ItemIndex := 2;
  end;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := (MessageDlg('Leave SQLBuilder Demonstration ?', mtConfirmation, [mbYes, mbNo], 0) = mrYes);
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Memo1Change(Sender: TObject);
begin
  sbRunQuery.enabled := (Memo1.Lines.Count > 0);
end;

procedure TForm1.sbRunQueryClick(Sender: TObject);
begin
  Query1.SQL.Assign(memo1.lines);
  PrepareAndRunQuery;
end;

procedure TForm1.sbBlobDemoClick(Sender: TObject);
begin
  f_TSQLBlobs := Nil;
  Try
    f_TSQLBlobs := Tf_TSQLBlobs.Create(Self);
    {Set SQLBuilder options to match options selected in checkboxes}
    SetBuilderOptions;

    {Set this to false as we will be loading a query from a database blob}
    SQLBuilder1.UseWizardOnShow := False;

    f_TSQLBlobs.SqlBuilder := SqlBuilder1;
    f_TSQLBlobs.showmodal;
  finally
    f_TSQLBlobs.free;
  end;
end;

end.
