unit blobmain;

{ ********** Main unit of TSQLBuilder Blob Database Demo *********** }
{ Demonstration of storing TSQLBuilder queries in a database blob field.
  This demo will create a new table in the DBDemos database called tsqlblob.db
  tsqlblob.db
    QueryKey : AutoInc;
    Title : Alpha[255];
    TheQueryStream : Blob;

  Primary Key = QueryKey;
  Other Index = ByTitle (Title);
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, sqlbuild, DB, DBTables, Grids, DBGrids, ExtCtrls, DBCtrls, Mask,
  Buttons;

type
  Tf_TSQLBlobs = class(TForm)
    dsTSQLBlobs: TDataSource;
    tblTSQLBlobs: TTable;
    DBGrid1: TDBGrid;
    EditTitle: TDBEdit;
    Memodescription: TDBMemo;
    DBNavigator1: TDBNavigator;
    Bevel1: TBevel;
    Label1: TLabel;
    Label2: TLabel;
    sbBuildSQL: TSpeedButton;
    BitBtn1: TBitBtn;
    tblTSQLBlobsTitle: TStringField;
    tblTSQLBlobsTheQueryStream: TBlobField;
    tblTSQLBlobsDescription: TMemoField;
    tblTSQLBlobsQueryKey: TIntegerField;
    procedure NavTSQLBlobsClick(Sender: TObject; Button: TNavigateBtn);
    procedure btnBuildQueryClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    function CreateNewTable:Boolean;
    { Creates a new table in dbdemos named tsqlblob.db }
  public
    { Public declarations }
    SQLBuilder:TSQLBuilder; {Reference copied from parent form}
  end;

var
  f_TSQLBlobs: Tf_TSQLBlobs;

implementation

{$R *.DFM}

function Tf_TSQLBlobs.CreateNewTable:Boolean;
{ Creates a new table in dbdemos named tsqlblob.db }
Var
  Atable : ttable;
begin
  result := false;
{  Try
    Atable:=ttable.create(Self);
    with Atable do
    Try
      active:=false;
      DatabaseName:=tblTSQLBlobs.Databasename;
      tablename:=tblTSQLBlobs.Tablename;
      exclusive:=true;
      FieldDefs.clear;
      IndexDefs.clear;
      FieldDefs.Add('QueryKey',ftAutoInc,0,false);
      FieldDefs.Add('Title',ftString,255,false);
      FieldDefs.Add('TheQueryStream',ftBlob,0,false);
      FieldDefs.Add('Description',ftMemo,240,false);
      CreateTable;
      Addindex('','QueryKey',[ixPrimary]+[ixUnique]+[]);
      Addindex('ByTitle','Title',[ixCaseInsensitive]+[]);
      result := True;
    finally
      Atable.free;
    end;
  except
    result := false;
  end;}
end;

procedure Tf_TSQLBlobs.NavTSQLBlobsClick(Sender: TObject;
  Button: TNavigateBtn);
begin
  Case Button of
    nbEdit :
      begin
        If EditTitle.Canfocus then EditTitle.Setfocus;
        tblTSQLBlobs.Edit;
      end;
    nbInsert :
      begin
        If EditTitle.Canfocus then EditTitle.Setfocus;
        tblTSQLBlobs.Insert;
      end;
    nbPost:
      begin
        If EditTitle.text = '' then
        begin
          MessageDlg('A title must be entered before the query can be saved', mtError, [mbOk], 0);
          If EditTitle.Canfocus then
            EditTitle.Setfocus;
          Abort;
        end;
      end;
  end;
end;

procedure Tf_TSQLBlobs.btnBuildQueryClick(Sender: TObject);
var
  MemStrm: TMemoryStream;
  WeChangedState:Boolean;
begin
  MemStrm := Nil;
  WeChangedState := False;
  Try
    MemStrm := TMemoryStream.Create;

    {If editing then load blob from current row into tsqlbuilder}
    If (tblTSQLBlobs.State <> dsInsert)
     and (Not (tblTSQLBlobs.BOF and tblTSQLBlobs.EOF)) then
    begin
      If not (dsTSQLBlobs.State in [dsInsert, dsEdit]) then
      begin
        tblTSQLBlobs.Edit;
        WeChangedState := True;
      end;
      tblTSQLBlobsTheQueryStream.SaveToStream(MemStrm);
      SQLBuilder.LoadFromStream(MemStrm);
    end;

    if SQLBuilder.Execute then
    begin
      {save the query defintion to the database}
      MemStrm.Clear;
      SQLBuilder.StoreToStream(MemStrm);
      tblTSQLBlobsTheQueryStream.LoadFromStream(MemStrm);
    end
    else
    begin
      {change state back if we put it in edit mode}
      If WeChangedState then
        tblTSQLBlobs.Cancel;
    end;
  finally
    MemStrm.Free;
  end;
end;

procedure Tf_TSQLBlobs.FormCreate(Sender: TObject);
begin
  Try
    tblTSQLBlobs.Active := False;
    tblTSQLBlobs.Databasename := ExtractFilepath(Application.Exename);
    tblTSQLBlobs.Active := True;
  except
    on E:EDatabaseError do
    begin
      Case MessageDlg('Unable to open tsqlblob.db. Create new table?', mtConfirmation, [mbYes, mbNo], 0) of
        mrYes:
          begin
            If CreateNewTable then
              tblTSQLBlobs.Active := True;
          end;
        mrNo: PostMessage(Handle, WM_CLOSE, 0, 0);
      end;
    end;
  end;
end;

end.
